@@ -1,8 +1,12 @@
 use crate::error::Error;
 use smart_default::SmartDefault;
 
+#[cfg(feature = "param_fromstr")]
+use strum_macros::EnumString;
+
 #[derive(Copy, Clone, Debug, PartialEq, Eq, SmartDefault)]
 #[cfg_attr(test, derive(proptest_derive::Arbitrary))]
+#[cfg_attr(feature = "param_fromstr", derive(EnumString))]
 pub enum Parity {
     #[default]
     None,
@@ -32,3 +36,19 @@ impl From<Parity> for u8 {
         }
     }
 }
+
+#[cfg(feature = "param_fromstr")]
+#[cfg(test)]
+mod test {
+    use super::*;
+    use core::str::FromStr;
+
+    #[test]
+    fn parses_parity() {
+        let none = Parity::from_str("None").unwrap();
+        assert_eq!(none, Parity::None);
+
+        let even = Parity::from_str("Even").unwrap();
+        assert_eq!(even, Parity::Even);
+    }
+}
